var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __typeError = (msg) => {
  throw TypeError(msg);
};
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
var __accessCheck = (obj, member, msg) => member.has(obj) || __typeError("Cannot " + msg);
var __privateGet = (obj, member, getter) => (__accessCheck(obj, member, "read from private field"), getter ? getter.call(obj) : member.get(obj));
var __privateAdd = (obj, member, value) => member.has(obj) ? __typeError("Cannot add the same private member more than once") : member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
var __privateSet = (obj, member, value, setter) => (__accessCheck(obj, member, "write to private field"), setter ? setter.call(obj, value) : member.set(obj, value), value);
var _transformer, _chalk, _chalkOriginalLevel, _options, _getContext;
const CTX = Symbol("context");
const DEFAULT_SIGNS = ["\u2714", "\u26A0", "\u2716"];
const DEFAULT_COLORS = ["green", "yellow", "red"];
const DEFAULT_FORMATTER_OPTIONS = {
  customizable: false,
  formatOptions: {
    colors: DEFAULT_COLORS,
    signs: DEFAULT_SIGNS
  }
};
class Formatter {
  /**
   * Creates an instance of the Formatter.
   * @param transformer The transformer object that handles input and result transformations.
   * @param chalk The `chalk` instance used to style the output.
   * @param options Optional formatting options.
   */
  constructor(transformer, chalk, options) {
    __privateAdd(this, _transformer);
    __privateAdd(this, _chalk);
    __privateAdd(this, _chalkOriginalLevel);
    __privateAdd(this, _options);
    /**
     * Gets the context to be passed to the transformation functions.
     * @param result The linting outcome of a commit.
     * @param formatOptions Additional formatting options.
     * @returns The format context.
     * @internal
     */
    __privateAdd(this, _getContext, (_formatOptions, reusable) => {
      if (_formatOptions == null ? void 0 : _formatOptions[CTX]) {
        return _formatOptions[CTX];
      }
      const formatOptions = __spreadValues(__spreadValues({}, __privateGet(this, _options).formatOptions), _formatOptions);
      const colors = formatOptions.colors || DEFAULT_COLORS;
      const signs = formatOptions.signs || DEFAULT_SIGNS;
      switch (formatOptions == null ? void 0 : formatOptions.color) {
        case void 0:
        case true: {
          __privateGet(this, _chalk).level = __privateGet(this, _chalkOriginalLevel);
          break;
        }
        case false: {
          __privateGet(this, _chalk).level = 0;
        }
      }
      const context = {
        formatOptions,
        chalk: __privateGet(this, _chalk),
        verbose: Boolean(formatOptions.verbose),
        getSign: (level) => {
          return context.chalk[colors[level]](signs[level]);
        }
      };
      if (reusable) {
        Object.defineProperty(context.formatOptions, CTX, {
          enumerable: false,
          value: __spreadProps(__spreadValues({}, context), { formatOptions: __spreadValues({}, formatOptions) })
        });
      }
      return context;
    });
    /**
     * Sets a new transformer for the formatter.
     * @param transformer The new transformer to be used.
     * @throws Will throw an error if the formatter is not customizable.
     * @throws Will throw an error if `transformInput` or `transformResult` are not functions.
     */
    __publicField(this, "setTransformer", (transformer) => {
      if (!__privateGet(this, _options).customizable) {
        throw new Error("formatter is not customizable.");
      }
      if (typeof transformer.transformInput !== "function") {
        throw new TypeError("`transformInput` is not a function.");
      }
      if (typeof transformer.transformResult !== "function") {
        throw new TypeError("`transformResult` is not a function.");
      }
      __privateGet(this, _transformer).transformInput = transformer.transformInput;
      __privateGet(this, _transformer).transformResult = transformer.transformResult;
    });
    /**
     * Sets the format options for the formatter.
     * @param formatOptions The format options to be applied.
     */
    __publicField(this, "setFormatOptions", (formatOptions) => {
      __privateGet(this, _options).formatOptions = __spreadValues(__spreadValues({}, __privateGet(this, _options).formatOptions), formatOptions);
    });
    /**
     * Formats the input of a linting result (commit message).
     * @param result The linting outcome.
     * @param formatOptions Optional format options.
     * @returns A string array with the formatted input.
     */
    __publicField(this, "formatInput", (result, formatOptions) => {
      const context = __privateGet(this, _getContext).call(this, formatOptions);
      const hasProblem = result.errors.length > 0 || result.warnings.length > 0;
      if (hasProblem || context.verbose) {
        return __privateGet(this, _transformer).transformInput(result, context);
      }
      return [];
    });
    /**
     * Formats the result of a linting outcome.
     * @param result The linting outcome.
     * @param formatOptions Optional format options.
     * @returns A string array with the formatted result.
     */
    __publicField(this, "formatResult", (result, formatOptions) => {
      const context = __privateGet(this, _getContext).call(this, formatOptions);
      const hasProblem = result.errors.length > 0 || result.warnings.length > 0;
      if (hasProblem || context.verbose) {
        return __privateGet(this, _transformer).transformResult(result, context);
      }
      return [];
    });
    /**
     * Formats the entire lint report into a string.
     * @param report The lint report containing multiple linting outcomes.
     * @param formatOptions Optional format options.
     * @returns A string with the formatted lint report.
     */
    __publicField(this, "format", (report, formatOptions) => {
      let formatted = "";
      const results = report.results;
      if (!results.length) {
        return formatted;
      }
      const context = __privateGet(this, _getContext).call(this, formatOptions, true);
      const {
        prefix = "",
        suffix = "",
        eachCommitPrefix = "",
        eachCommitSuffix = "",
        separatorBetweenCommits = "\n",
        separatorBetweenInputAndResult = "\n",
        finalizeFormatResult
      } = context.formatOptions;
      const lastIndex = results.length - 1;
      const summary = {
        totalCommit: results.length,
        totalError: 0,
        totalWarning: 0
      };
      for (let i = 0; i < results.length; i++) {
        const result = results[i];
        summary.totalError += result.errors.length;
        summary.totalWarning += result.warnings.length;
        const formattedInput = this.formatInput(
          result,
          context.formatOptions
        ).join("");
        const formattedResult = this.formatResult(
          result,
          context.formatOptions
        ).join("");
        formatted += eachCommitPrefix;
        formatted += formattedInput;
        if (formattedInput) {
          formatted += separatorBetweenInputAndResult;
        }
        formatted += formattedResult;
        formatted += eachCommitSuffix;
        const isFormattedNotEmpty = Boolean(formattedInput || formattedResult);
        if (isFormattedNotEmpty && i !== lastIndex) {
          formatted += separatorBetweenCommits;
        }
      }
      formatted = `${prefix}${formatted}${suffix}`;
      if (finalizeFormatResult) {
        formatted = finalizeFormatResult(formatted, context, summary);
      }
      return formatted;
    });
    __privateSet(this, _transformer, transformer);
    __privateSet(this, _chalk, chalk);
    __privateSet(this, _chalkOriginalLevel, chalk.level);
    __privateSet(this, _options, options || DEFAULT_FORMATTER_OPTIONS);
  }
  /** Whether the formatter is customizable. */
  get isCustomizable() {
    return Boolean(__privateGet(this, _options).customizable);
  }
}
_transformer = new WeakMap();
_chalk = new WeakMap();
_chalkOriginalLevel = new WeakMap();
_options = new WeakMap();
_getContext = new WeakMap();
export {
  Formatter
};
